@@ -1,9 +1,9 @@
 /* Test of lstat() function.
-   Copyright (C) 2008-2020 Free Software Foundation, Inc.
+   Copyright (C) 2008-2023 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -32,14 +32,14 @@ test_lstat_func (int (*func) (char const *, struct stat *), bool print)
   ASSERT (func (".", &st1) == 0);
   ASSERT (func ("./", &st2) == 0);
 #if !(defined _WIN32 && !defined __CYGWIN__ && !_GL_WINDOWS_STAT_INODES)
-  ASSERT (SAME_INODE (st1, st2));
+  ASSERT (psame_inode (&st1, &st2));
 #endif
   ASSERT (S_ISDIR (st1.st_mode));
   ASSERT (S_ISDIR (st2.st_mode));
   ASSERT (func ("/", &st1) == 0);
   ASSERT (func ("///", &st2) == 0);
 #if !(defined _WIN32 && !defined __CYGWIN__ && !_GL_WINDOWS_STAT_INODES)
-  ASSERT (SAME_INODE (st1, st2));
+  ASSERT (psame_inode (&st1, &st2));
 #endif
   ASSERT (S_ISDIR (st1.st_mode));
   ASSERT (S_ISDIR (st2.st_mode));
@@ -90,7 +90,7 @@ test_lstat_func (int (*func) (char const *, struct stat *), bool print)
   ASSERT (S_ISDIR (st1.st_mode));
   ASSERT (S_ISDIR (st2.st_mode));
 #if !(defined _WIN32 && !defined __CYGWIN__ && !_GL_WINDOWS_STAT_INODES)
-  ASSERT (SAME_INODE (st1, st2));
+  ASSERT (psame_inode (&st1, &st2));
 #endif
 
   ASSERT (func (BASE "link2", &st1) == 0);
